@@ -15,15 +15,19 @@ fct.eta2 <- function(vec,x,weights) {   ## pb avec les poids
     if (is.table(X)) X <- matrix(as.vector(X),nrow(X),dimnames=dimnames(X))
     if (is.null(rownames(X))) rownames(X) <- 1:nrow(X)
     if (is.null(colnames(X))) colnames(X) <- colnames(X, do.NULL = FALSE,prefix="V")
+    if (!is.null(row.sup) & !is.numeric(row.sup)) row.sup<- which(rownames(X)%in%row.sup)
+    if (!is.null(col.sup) & !is.numeric(col.sup)) col.sup<- which(colnames(X)%in%col.sup)
+    if (!is.null(quali.sup) & !is.numeric(quali.sup)) quali.sup<- which(colnames(X)%in%quali.sup)
+    if (!is.null(quanti.sup) & !is.numeric(quanti.sup)) quanti.sup<- which(colnames(X)%in%quanti.sup)
 	X <- as.data.frame(X)
     is.quali <- which(!unlist(lapply(X,is.numeric)))
     X[,is.quali] <- lapply(X[,is.quali,drop=FALSE],as.factor)
-    for (i in is.quali) X[,i]=as.factor(X[,i])
+    for (i in is.quali) X[,i] <- as.factor(X[,i])
 	X <- droplevels(X)
     Xtot <- X
-    if (any(!sapply(X, is.numeric))) {
-      auxi = NULL
-      for (j in (1:ncol(X))[!((1:ncol(X))%in%quali.sup)]) if (!is.numeric(X[,j])) auxi = c(auxi,colnames(X)[j])
+    if (any(!unlist(lapply(X, is.numeric)))) {
+      auxi <- NULL
+      for (j in (1:ncol(X))[!((1:ncol(X))%in%quali.sup)]) if (!is.numeric(X[,j])) auxi <- c(auxi,colnames(X)[j])
       if (!is.null(auxi)) stop(paste("\nThe following variables are not quantitative: ", auxi))
     }
     if (!inherits(X, "data.frame")) stop("X is not a data.frame")
@@ -38,7 +42,7 @@ fct.eta2 <- function(vec,x,weights) {   ## pb avec les poids
  	  X <- X[,-which(apply(X,2,sum)==0),drop=FALSE]
 	}
 ### 3 lignes rajoutees
-    if (is.null(row.w)) row.w = rep(1,nrow(X))
+    if (is.null(row.w)) row.w <- rep(1,nrow(X))
 	row.w.init <- row.w
     if (length(row.w)!=nrow(X)) stop("length of vector row.w should be the number of active rows")
     total <- sum(X*row.w)
@@ -67,13 +71,13 @@ fct.eta2 <- function(vec,x,weights) {   ## pb avec les poids
     cos2.col <- coord.col^2/dist2.col
     colnames(coord.col) <- colnames(contrib.col) <- colnames(cos2.col) <- paste("Dim", 1:length(eig))
     rownames(coord.col) <- rownames(contrib.col) <- rownames(cos2.col) <- attributes(X)$names
-dist2.row <- rowSums(t(t(Tc^2)*marge.col))
+    dist2.row <- rowSums(t(t(Tc^2)*marge.col))
     contrib.row <- t(t(coord.row^2*marge.row)/eig)
     cos2.row <- coord.row^2/dist2.row
     colnames(coord.row) <- colnames(contrib.row) <- colnames(cos2.row) <- paste("Dim", 1:length(eig))
     rownames(coord.row) <- rownames(contrib.row) <- rownames(cos2.row) <- attributes(X)$row.names
-    inertia.row = marge.row*dist2.row
-    inertia.col = marge.col*dist2.col
+    inertia.row <- marge.row*dist2.row
+    inertia.col <- marge.col*dist2.col
     names(inertia.col) <- attributes(coord.col)$row.names
     names(inertia.row) <- attributes(coord.row)$row.names
     
@@ -85,11 +89,17 @@ dist2.row <- rowSums(t(t(Tc^2)*marge.col))
   if (!is.null(row.sup)){
     X.row.sup <- as.data.frame(Xtot[row.sup,])
     if ((!is.null(col.sup))||(!is.null(quanti.sup))||(!is.null(quali.sup))) X.row.sup <- as.data.frame(X.row.sup[,-c(col.sup,quanti.sup,quali.sup)])
-    somme.row <- rowSums(X.row.sup)
-    X.row.sup <- X.row.sup/somme.row
-    coord.row.sup <- crossprod(t(as.matrix(X.row.sup)),V)
+	somme.row <- rowSums(X.row.sup)
+    if (is.null(excl)) {
+	  X.row.sup <- X.row.sup/somme.row
+	  coord.row.sup <- crossprod(t(as.matrix(X.row.sup)),V)
+	} else {
+	  X.row.sup <- t(t(X.row.sup/somme.row)/colSums(F))-1
+      coord.row.sup <- crossprod(t(as.matrix(X.row.sup)),V*marge.col)  ## difference with excl=NULL
+	}
 # modif
-    dist2.row <- rowSums(t((t(X.row.sup)-marge.col)^2/marge.col))
+    if (is.null(excl)) dist2.row <- rowSums(t((t(X.row.sup)-marge.col)^2/marge.col))
+	else dist2.row <- rowSums(t(t(X.row.sup^2)*marge.col))
 #dist2.row <- rowSums(sweep(sweep(X.row.sup,2,marge.col,FUN="-")^2,2,1/marge.col,FUN="*"))
     cos2.row.sup <- coord.row.sup^2/dist2.row
     coord.row.sup <- coord.row.sup[, 1:ncp,drop=FALSE]
@@ -143,10 +153,17 @@ dist2.col <- colSums((X.col.sup-marge.row)^2/marge.row)
       if (!is.null(row.sup)) X.del <- as.data.frame(Xtot[-row.sup,-c(col.sup,quanti.sup,quali.sup)])
 	  else X.del <- Xtot[,-c(col.sup,quanti.sup,quali.sup)]
 	  X.quali.sup <- NULL
-	  for (j in 1:length(quali.sup)){
-	    if (!is.null(row.sup)) X.quali.sup <- rbind(X.quali.sup,matrix(unlist(by(X.del,Xtot[-row.sup,quali.sup[j]],colSums)),ncol=ncol(X.del),byrow=T))
-	    else X.quali.sup <- rbind(X.quali.sup,matrix(unlist(by(X.del,Xtot[,quali.sup[j]],colSums)),ncol=ncol(X.del),byrow=T))
-	  }
+	  # for (j in 1:length(quali.sup)){
+	    # if (!is.null(row.sup)) X.quali.sup <- rbind(X.quali.sup,matrix(unlist(by(X.del,Xtot[-row.sup,quali.sup[j]],colSums)),ncol=ncol(X.del),byrow=T))
+	    # else X.quali.sup <- rbind(X.quali.sup,matrix(unlist(by(X.del,Xtot[,quali.sup[j]],colSums)),ncol=ncol(X.del),byrow=T))
+	  # }
+      Xtot2 <- Xtot
+      if (!is.null(row.sup))  Xtot2 <- Xtot[-row.sup,] 
+      for (j in 1:length(quali.sup)) {
+        Xtot2[,quali.sup[j]] <- droplevels(Xtot2[,quali.sup[j]] , reorder=FALSE)  
+        X.quali.sup <- rbind(X.quali.sup, matrix(unlist(by(X.del, 
+              Xtot2[, quali.sup[j]], colSums)), ncol = ncol(X.del), byrow = T))
+      }
       somme.quali <- rowSums(X.quali.sup)
       X.quali.sup <- X.quali.sup/somme.quali
       coord.quali.sup <- crossprod(t(as.matrix(X.quali.sup)),V)
@@ -154,66 +171,22 @@ dist2.col <- colSums((X.col.sup-marge.row)^2/marge.row)
       cos2.quali.sup <- coord.quali.sup^2/dist2.quali
       coord.quali.sup <- coord.quali.sup[, 1:ncp,drop=FALSE]
       cos2.quali.sup <- cos2.quali.sup[, 1:ncp,drop=FALSE]
-      rownames(coord.quali.sup) <- rownames(cos2.quali.sup) <- paste(rep(colnames(Xtot)[quali.sup],lapply(Xtot[,quali.sup,drop=FALSE],nlevels)) , unlist(lapply(Xtot[,quali.sup,drop=FALSE],levels)),sep=".")
+      rownames(coord.quali.sup) <- rownames(cos2.quali.sup) <- paste(rep(colnames(Xtot2)[quali.sup],lapply(Xtot2[, quali.sup, drop = FALSE], nlevels)), unlist(lapply(Xtot2[, quali.sup, drop = FALSE], levels)),sep = ".") 
       colnames(coord.quali.sup) <- colnames(cos2.quali.sup) <- paste("Dim", 1:ncp)
       res$quali.sup <- list(coord = coord.quali.sup, cos2 = cos2.quali.sup)
 
-        if (!is.null(row.sup)) Zqs <- tab.disjonctif(Xtot[-row.sup,quali.sup])
-		else Zqs <- tab.disjonctif(Xtot[,quali.sup])
+        Zqs <- tab.disjonctif(Xtot2[, quali.sup])
         Nj <- colSums(Zqs * row.w)
         Nj <- colSums(Zqs * marge.row)*total
         if (total>1) coef <- sqrt(Nj * ((total - 1)/(total - Nj)))
 		else coef <- sqrt(Nj)
         res$quali.sup$v.test <- res$quali.sup$coord*coef
 
-
-##########
-    # if (!is.null(row.sup)) X.del <- as.data.frame(Xtot[-row.sup,-c(col.sup,quanti.sup,quali.sup)])
-	# else X.del <- Xtot[,-c(col.sup,quanti.sup,quali.sup)]
-	# X.quali.sup <- NULL
-	# for (j in 1:length(quali.sup)){
-	  # if (!is.null(row.sup)) X.quali.sup <- rbind(X.quali.sup,matrix(unlist(by(X.del,Xtot[-row.sup,quali.sup[j]],colSums)),ncol=ncol(X.del),byrow=T))
-	  # else X.quali.sup <- rbind(X.quali.sup,matrix(unlist(by(X.del,Xtot[,quali.sup[j]],colSums)),ncol=ncol(X.del),byrow=T))
-	# }
-# print(X.quali.sup)
-    # somme.quali <- rowSums(X.quali.sup)
-    # X.quali.sup <- X.quali.sup/somme.quali
-    # coord.quali.sup <- crossprod(t(as.matrix(X.quali.sup)),V)
-    # dist2.quali <- rowSums(t((t(X.quali.sup)-marge.col)^2/marge.col))
-    # cos2.quali.sup <- coord.quali.sup^2/dist2.quali
-    # coord.quali.sup <- coord.quali.sup[, 1:ncp,drop=FALSE]
-    # cos2.quali.sup <- cos2.quali.sup[, 1:ncp,drop=FALSE]
-    # colnames(coord.quali.sup) <- colnames(cos2.quali.sup) <- paste("Dim", 1:ncp)
-    # rownames(coord.quali.sup) <- rownames(cos2.quali.sup) <- rownames(X.quali.sup)
-    # res.quali.sup <- list(coord = coord.quali.sup, cos2 = cos2.quali.sup)
-    # res$quali.sup <- res.quali.sup
-    # res$call$quali.sup <- quali.sup
-
-##########
-
-
-
-
-
-
-
-
-
-		eta2 = matrix(NA, length(quali.sup), ncp)
-         # if (is.null(row.sup)) {
-		   # for (i in 1:ncp)  eta2[, i] <- unlist(lapply(as.data.frame(Xtot[, quali.sup]),fct.eta2,res$row$coord[,i],weights=marge.row))
-		 # } else {
-		   # for (i in 1:ncp)  eta2[, i] <- unlist(lapply(as.data.frame(Xtot[-row.sup, quali.sup]),fct.eta2,res$row$coord[,i],weights=marge.row))
-		 # }	  
-
-         if (is.null(row.sup)) {
-		   eta2 <- sapply(as.data.frame(Xtot[, quali.sup,drop=FALSE]),fct.eta2,res$row$coord,weights=marge.row)
-		 } else {
-		   eta2 <- sapply(as.data.frame(Xtot[-row.sup, quali.sup,drop=FALSE]),fct.eta2,res$row$coord,weights=marge.row)
-		 }
+		eta2 <- matrix(NA, length(quali.sup), ncp)
+        eta2 <- sapply(as.data.frame(Xtot2[, quali.sup, drop = FALSE]), fct.eta2, res$row$coord, weights = marge.row)
 		eta2 <- t(as.matrix(eta2,ncol=ncp))
-        colnames(eta2) = paste("Dim", 1:ncp)
-        rownames(eta2) = colnames(Xtot)[quali.sup]
+        colnames(eta2) <- paste("Dim", 1:ncp)
+        rownames(eta2) <- colnames(Xtot)[quali.sup]
 
         res$quali.sup$eta2 <- eta2
         res$call$quali.sup <- quali.sup
